import processing.core.*; 
import processing.data.*; 
import processing.event.*; 
import processing.opengl.*; 

import java.util.HashMap; 
import java.util.ArrayList; 
import java.io.File; 
import java.io.BufferedReader; 
import java.io.PrintWriter; 
import java.io.InputStream; 
import java.io.OutputStream; 
import java.io.IOException; 

public class lerw extends PApplet {

// want to draw a srw on a square box of size (w, h)

int w = 600; int h = 600;
int[] xc = new int[0]; // array of x-coords of the walk
int[] yc = new int[0]; // y-coords

// coords of the [tentative] loop-erased random walk
int[] lexc = new int[0];
int[] leyc = new int[0];

int numSteps = 0;
int numLerwSteps = 0;
boolean isStopped = false;
public void addPoint(int x, int y) {
   xc = append(xc, x); yc = append(yc, y); 
   numSteps++;
}

public void addPointToLerw(int x, int y) {
   lexc = append(lexc, x); leyc = append(leyc, y); 
   numLerwSteps++;
}

// adds one random step to the random walk
// stop it when it exits
public void moveRandomWalk() {
    if (isStopped) return; 
    int currentX = xc[numSteps - 1];
    int currentY = yc[numSteps - 1];
    if (currentX < 0 || currentX >= w
         || currentY < 0 || currentY >= h) {
        isStopped = true; return;        
     }
    // unifs [0, 1]
    float u1 = random(1.0f); float u2 = random(1.0f);
    int newX = currentX; int newY = currentY;
    if (u1 < 0.5f) { // move horizontal w.p. 1/2
      if (u2 < 0.5f) newX++; else newX--;
    }
    else { // move vertical otherwise
      if (u2 < 0.5f) newY++; else newY--;
    }
    addPoint(newX, newY);
    updateLerw(newX, newY);
    
}

public void updateLerw(int newX, int newY) {
   int firstOcc = findFirstOccOfInLerw(newX, newY);
   if (firstOcc == -1) {
     addPointToLerw(newX, newY);
   }
   else {
      // selects the elements [0, ..., firstOcc]
      lexc = subset(lexc, 0, firstOcc + 1);  
      leyc = subset(leyc, 0, firstOcc + 1);
      numLerwSteps = firstOcc + 1;
   }
}

// look for the first time (x, y) appears in lerw
// -1 if it doesn't exist
public int findFirstOccOfInLerw(int x, int y) {
   for (int i = 0; i < numLerwSteps; i++) {
       if (lexc[i] == x && leyc[i] == y) 
         return i;
    }
    return -1;
}

int ew; int eh;
// the beginning
public void setup() {
  
  ew = max(width / w, 2);
  eh = max(height / h, 2);
  init();
}

public void init() {
  xc = new int[0]; yc = new int[0];
  lexc = new int[0]; leyc = new int[0];
  numSteps = 0; numLerwSteps = 0;
  addPoint(w / 2, h / 2); // add a point in midscreen
  isStopped = false;
}

public void draw() {
    noStroke();
   for (int i = 0; i < 50; i++) moveRandomWalk();
   background(255); fill(210, 210, 245);
   for (int i = 0; i < numSteps; i++) {
       int x = xc[i]; int y = yc[i];
       ellipse(x * ew, y * eh, ew / 2, eh / 2); 
   }
   fill(255, 0, 0);
   for (int i = 0; i < numLerwSteps; i++) {
       int x = lexc[i]; int y = leyc[i];
       ellipse(x * ew, y * eh, ew / 2, eh / 2); 
   }
}

public void keyPressed() {
   if (key == ' ') { init(); } 
}
  public void settings() {  fullScreen(); }
  static public void main(String[] passedArgs) {
    String[] appletArgs = new String[] { "lerw" };
    if (passedArgs != null) {
      PApplet.main(concat(appletArgs, passedArgs));
    } else {
      PApplet.main(appletArgs);
    }
  }
}
